@@ -1,53 +1,35 @@
 open! Import
 module Array = Array0
+include Bytes_intf
 
 let stage = Staged.stage
 
 module T = struct
-  type t = bytes [@@deriving_inline sexp, sexp_grammar]
+  type t = bytes [@@deriving_inline globalize, sexp, sexp_grammar]
 
-  let t_of_sexp = (bytes_of_sexp : Ppx_sexp_conv_lib.Sexp.t -> t)
-  let sexp_of_t = (sexp_of_bytes : t -> Ppx_sexp_conv_lib.Sexp.t)
-
-  let (t_sexp_grammar : Ppx_sexp_conv_lib.Sexp.Private.Raw_grammar.t) =
-    let (_the_generic_group : Ppx_sexp_conv_lib.Sexp.Private.Raw_grammar.generic_group) =
-      { implicit_vars = [ "bytes" ]
-      ; ggid = "\146e\023\249\235eE\139c\132W\195\137\129\235\025"
-      ; types = [ "t", Implicit_var 0 ]
-      }
-    in
-    let (_the_group : Ppx_sexp_conv_lib.Sexp.Private.Raw_grammar.group) =
-      { gid = Ppx_sexp_conv_lib.Lazy_group_id.create ()
-      ; apply_implicit = [ bytes_sexp_grammar ]
-      ; generic_group = _the_generic_group
-      ; origin = "bytes.ml.T"
-      }
-    in
-    let (t_sexp_grammar : Ppx_sexp_conv_lib.Sexp.Private.Raw_grammar.t) =
-      Ref ("t", _the_group)
-    in
-    t_sexp_grammar
-  ;;
+  let (globalize : t -> t) = (globalize_bytes : t -> t)
+  let t_of_sexp = (bytes_of_sexp : Sexplib0.Sexp.t -> t)
+  let sexp_of_t = (sexp_of_bytes : t -> Sexplib0.Sexp.t)
+  let (t_sexp_grammar : t Sexplib0.Sexp_grammar.t) = bytes_sexp_grammar
 
   [@@@end]
 
   include Bytes0
 
   let module_name = "Base.Bytes"
-  let pp fmt t = Caml.Format.fprintf fmt "%S" (to_string t)
+  let pp fmt t = Stdlib.Format.fprintf fmt "%S" (to_string t)
 end
 
 include T
 
 module To_bytes = Blit.Make (struct
-    include T
+  include T
 
-    let create ~len = create len
-  end)
+  let create ~len = create len
+end)
 
 include To_bytes
 include Comparator.Make (T)
-include Comparable.Validate (T)
 include Pretty_printer.Register_pp (T)
 
 (* Open replace_polymorphic_compare after including functor instantiations so they do not
@@ -171,6 +153,26 @@ let contains ?pos ?len t char =
   loop pos
 ;;
 
+module Utf8 = struct
+  let set = set_uchar_utf_8
+end
+
+module Utf16le = struct
+  let set = set_uchar_utf_16le
+end
+
+module Utf16be = struct
+  let set = set_uchar_utf_16be
+end
+
+module Utf32le = struct
+  let set = set_uchar_utf_32le
+end
+
+module Utf32be = struct
+  let set = set_uchar_utf_32be
+end
+
 (* Include type-specific [Replace_polymorphic_compare] at the end, after
    including functor application that could shadow its definitions. This is
    here so that efficient versions of the comparison functions are exported by
