@@ -53,16 +53,16 @@ module type S0 = sig
 
   (** [fold t ~init ~f] returns [f (... f (f (f init e1) e2) e3 ...) en], where [e1..en]
       are the elements of [t]. *)
-  val fold : t -> init:'accum -> f:('accum -> elt -> 'accum) -> 'accum
+  val fold : t -> init:'acc -> f:('acc -> elt -> 'acc) -> 'acc
 
   (** [fold_result t ~init ~f] is a short-circuiting version of [fold] that runs in the
       [Result] monad.  If [f] returns an [Error _], that value is returned without any
       additional invocations of [f]. *)
   val fold_result
     :  t
-    -> init:'accum
-    -> f:('accum -> elt -> ('accum, 'e) Result.t)
-    -> ('accum, 'e) Result.t
+    -> init:'acc
+    -> f:('acc -> elt -> ('acc, 'e) Result.t)
+    -> ('acc, 'e) Result.t
 
   (** [fold_until t ~init ~f ~finish] is a short-circuiting version of [fold]. If [f]
       returns [Stop _] the computation ceases and results in that value. If [f] returns
@@ -95,9 +95,9 @@ module type S0 = sig
       ]} *)
   val fold_until
     :  t
-    -> init:'accum
-    -> f:('accum -> elt -> ('accum, 'final) Continue_or_stop.t)
-    -> finish:('accum -> 'final)
+    -> init:'acc
+    -> f:('acc -> elt -> ('acc, 'final) Continue_or_stop.t)
+    -> finish:('acc -> 'final)
     -> 'final
 
   (** Returns [true] if and only if there exists an element for which the provided
@@ -146,16 +146,16 @@ module type S0_phantom = sig
 
   (** [fold t ~init ~f] returns [f (... f (f (f init e1) e2) e3 ...) en], where [e1..en]
       are the elements of [t]. *)
-  val fold : _ t -> init:'accum -> f:('accum -> elt -> 'accum) -> 'accum
+  val fold : _ t -> init:'acc -> f:('acc -> elt -> 'acc) -> 'acc
 
   (** [fold_result t ~init ~f] is a short-circuiting version of [fold] that runs in the
       [Result] monad.  If [f] returns an [Error _], that value is returned without any
       additional invocations of [f]. *)
   val fold_result
     :  _ t
-    -> init:'accum
-    -> f:('accum -> elt -> ('accum, 'e) Result.t)
-    -> ('accum, 'e) Result.t
+    -> init:'acc
+    -> f:('acc -> elt -> ('acc, 'e) Result.t)
+    -> ('acc, 'e) Result.t
 
   (** [fold_until t ~init ~f ~finish] is a short-circuiting version of [fold]. If [f]
       returns [Stop _] the computation ceases and results in that value. If [f] returns
@@ -188,9 +188,9 @@ module type S0_phantom = sig
       ]} *)
   val fold_until
     :  _ t
-    -> init:'accum
-    -> f:('accum -> elt -> ('accum, 'final) Continue_or_stop.t)
-    -> finish:('accum -> 'final)
+    -> init:'acc
+    -> f:('acc -> elt -> ('acc, 'final) Continue_or_stop.t)
+    -> finish:('acc -> 'final)
     -> 'final
 
   (** Returns [true] if and only if there exists an element for which the provided
@@ -239,16 +239,16 @@ module type S1 = sig
 
   (** [fold t ~init ~f] returns [f (... f (f (f init e1) e2) e3 ...) en], where [e1..en]
       are the elements of [t]  *)
-  val fold : 'a t -> init:'accum -> f:('accum -> 'a -> 'accum) -> 'accum
+  val fold : 'a t -> init:'acc -> f:('acc -> 'a -> 'acc) -> 'acc
 
   (** [fold_result t ~init ~f] is a short-circuiting version of [fold] that runs in the
       [Result] monad.  If [f] returns an [Error _], that value is returned without any
       additional invocations of [f]. *)
   val fold_result
     :  'a t
-    -> init:'accum
-    -> f:('accum -> 'a -> ('accum, 'e) Result.t)
-    -> ('accum, 'e) Result.t
+    -> init:'acc
+    -> f:('acc -> 'a -> ('acc, 'e) Result.t)
+    -> ('acc, 'e) Result.t
 
   (** [fold_until t ~init ~f ~finish] is a short-circuiting version of [fold]. If [f]
       returns [Stop _] the computation ceases and results in that value. If [f] returns
@@ -281,9 +281,9 @@ module type S1 = sig
       ]} *)
   val fold_until
     :  'a t
-    -> init:'accum
-    -> f:('accum -> 'a -> ('accum, 'final) Continue_or_stop.t)
-    -> finish:('accum -> 'final)
+    -> init:'acc
+    -> f:('acc -> 'a -> ('acc, 'final) Continue_or_stop.t)
+    -> finish:('acc -> 'final)
     -> 'final
 
   (** Returns [true] if and only if there exists an element for which the provided
@@ -298,6 +298,7 @@ module type S1 = sig
   val count : 'a t -> f:('a -> bool) -> int
 
   (** Returns the sum of [f i] for all [i] in the container. *)
+
   val sum : (module Summable with type t = 'sum) -> 'a t -> f:('a -> 'sum) -> 'sum
 
   (** Returns as an [option] the first element for which [f] evaluates to true. *)
@@ -319,7 +320,7 @@ module type S1 = sig
   val max_elt : 'a t -> compare:('a -> 'a -> int) -> 'a option
 end
 
-module type S1_phantom_invariant = sig
+module type S1_phantom = sig
   type ('a, 'phantom) t
 
   (** Checks whether the provided element is there, using [equal]. *)
@@ -331,16 +332,16 @@ module type S1_phantom_invariant = sig
 
   (** [fold t ~init ~f] returns [f (... f (f (f init e1) e2) e3 ...) en], where [e1..en]
       are the elements of [t]. *)
-  val fold : ('a, _) t -> init:'accum -> f:('accum -> 'a -> 'accum) -> 'accum
+  val fold : ('a, _) t -> init:'acc -> f:('acc -> 'a -> 'acc) -> 'acc
 
   (** [fold_result t ~init ~f] is a short-circuiting version of [fold] that runs in the
       [Result] monad.  If [f] returns an [Error _], that value is returned without any
       additional invocations of [f]. *)
   val fold_result
     :  ('a, _) t
-    -> init:'accum
-    -> f:('accum -> 'a -> ('accum, 'e) Result.t)
-    -> ('accum, 'e) Result.t
+    -> init:'acc
+    -> f:('acc -> 'a -> ('acc, 'e) Result.t)
+    -> ('acc, 'e) Result.t
 
   (** [fold_until t ~init ~f ~finish] is a short-circuiting version of [fold]. If [f]
       returns [Stop _] the computation ceases and results in that value. If [f] returns
@@ -373,9 +374,9 @@ module type S1_phantom_invariant = sig
       ]} *)
   val fold_until
     :  ('a, _) t
-    -> init:'accum
-    -> f:('accum -> 'a -> ('accum, 'final) Continue_or_stop.t)
-    -> finish:('accum -> 'final)
+    -> init:'acc
+    -> f:('acc -> 'a -> ('acc, 'final) Continue_or_stop.t)
+    -> finish:('acc -> 'final)
     -> 'final
 
   (** Returns [true] if and only if there exists an element for which the provided
@@ -411,91 +412,153 @@ module type S1_phantom_invariant = sig
   val max_elt : ('a, _) t -> compare:('a -> 'a -> int) -> 'a option
 end
 
-module type S1_phantom = sig
-  type ('a, +'phantom) t
-
-  include S1_phantom_invariant with type ('a, 'phantom) t := ('a, 'phantom) t
-end
-
 module type Generic = sig
-  type 'a t
+  type ('a, 'phantom1, 'phantom2) t
   type 'a elt
 
-  val length : _ t -> int
-  val is_empty : _ t -> bool
-  val iter : 'a t -> f:('a elt -> unit) -> unit
-  val fold : 'a t -> init:'accum -> f:('accum -> 'a elt -> 'accum) -> 'accum
+  val length : (_, _, _) t -> int
+  val is_empty : (_, _, _) t -> bool
+  val mem : ('a, _, _) t -> 'a elt -> equal:('a elt -> 'a elt -> bool) -> bool
+  val iter : ('a, _, _) t -> f:('a elt -> unit) -> unit
+  val fold : ('a, _, _) t -> init:'acc -> f:('acc -> 'a elt -> 'acc) -> 'acc
 
   val fold_result
-    :  'a t
-    -> init:'accum
-    -> f:('accum -> 'a elt -> ('accum, 'e) Result.t)
-    -> ('accum, 'e) Result.t
+    :  ('a, _, _) t
+    -> init:'acc
+    -> f:('acc -> 'a elt -> ('acc, 'e) Result.t)
+    -> ('acc, 'e) Result.t
 
   val fold_until
-    :  'a t
-    -> init:'accum
-    -> f:('accum -> 'a elt -> ('accum, 'final) Continue_or_stop.t)
-    -> finish:('accum -> 'final)
+    :  ('a, _, _) t
+    -> init:'acc
+    -> f:('acc -> 'a elt -> ('acc, 'final) Continue_or_stop.t)
+    -> finish:('acc -> 'final)
     -> 'final
 
-  val exists : 'a t -> f:('a elt -> bool) -> bool
-  val for_all : 'a t -> f:('a elt -> bool) -> bool
-  val count : 'a t -> f:('a elt -> bool) -> int
-  val sum : (module Summable with type t = 'sum) -> 'a t -> f:('a elt -> 'sum) -> 'sum
-  val find : 'a t -> f:('a elt -> bool) -> 'a elt option
-  val find_map : 'a t -> f:('a elt -> 'b option) -> 'b option
-  val to_list : 'a t -> 'a elt list
-  val to_array : 'a t -> 'a elt array
-  val min_elt : 'a t -> compare:('a elt -> 'a elt -> int) -> 'a elt option
-  val max_elt : 'a t -> compare:('a elt -> 'a elt -> int) -> 'a elt option
-end
-
-module type Generic_phantom = sig
-  type ('a, 'phantom) t
-  type 'a elt
-
-  val length : (_, _) t -> int
-  val is_empty : (_, _) t -> bool
-  val iter : ('a, _) t -> f:('a elt -> unit) -> unit
-  val fold : ('a, _) t -> init:'accum -> f:('accum -> 'a elt -> 'accum) -> 'accum
-
-  val fold_result
-    :  ('a, _) t
-    -> init:'accum
-    -> f:('accum -> 'a elt -> ('accum, 'e) Result.t)
-    -> ('accum, 'e) Result.t
-
-  val fold_until
-    :  ('a, _) t
-    -> init:'accum
-    -> f:('accum -> 'a elt -> ('accum, 'final) Continue_or_stop.t)
-    -> finish:('accum -> 'final)
-    -> 'final
-
-  val exists : ('a, _) t -> f:('a elt -> bool) -> bool
-  val for_all : ('a, _) t -> f:('a elt -> bool) -> bool
-  val count : ('a, _) t -> f:('a elt -> bool) -> int
+  val exists : ('a, _, _) t -> f:('a elt -> bool) -> bool
+  val for_all : ('a, _, _) t -> f:('a elt -> bool) -> bool
+  val count : ('a, _, _) t -> f:('a elt -> bool) -> int
 
   val sum
     :  (module Summable with type t = 'sum)
-    -> ('a, _) t
+    -> ('a, _, _) t
     -> f:('a elt -> 'sum)
     -> 'sum
 
-  val find : ('a, _) t -> f:('a elt -> bool) -> 'a elt option
-  val find_map : ('a, _) t -> f:('a elt -> 'b option) -> 'b option
-  val to_list : ('a, _) t -> 'a elt list
-  val to_array : ('a, _) t -> 'a elt array
-  val min_elt : ('a, _) t -> compare:('a elt -> 'a elt -> int) -> 'a elt option
-  val max_elt : ('a, _) t -> compare:('a elt -> 'a elt -> int) -> 'a elt option
+  val find : ('a, _, _) t -> f:('a elt -> bool) -> 'a elt option
+  val find_map : ('a, _, _) t -> f:('a elt -> 'b option) -> 'b option
+  val to_list : ('a, _, _) t -> 'a elt list
+  val to_array : ('a, _, _) t -> 'a elt array
+  val min_elt : ('a, _, _) t -> compare:('a elt -> 'a elt -> int) -> 'a elt option
+  val max_elt : ('a, _, _) t -> compare:('a elt -> 'a elt -> int) -> 'a elt option
+end
+
+module type S0_with_creators = sig
+  include S0
+
+  val of_list : elt list -> t
+  val of_array : elt array -> t
+
+  (** E.g., [append (of_list [a; b]) (of_list [c; d; e])] is [of_list [a; b; c; d; e]] *)
+  val append : t -> t -> t
+
+  (** Concatenates a nested container. The elements of the inner containers are
+      concatenated together in order to give the result. *)
+  val concat : t list -> t
+
+  (** [map f (of_list [a1; ...; an])] applies [f] to [a1], [a2], ..., [an], in order, and
+      builds a result equivalent to [of_list [f a1; ...; f an]]. *)
+  val map : t -> f:(elt -> elt) -> t
+
+  (** [filter t ~f] returns all the elements of [t] that satisfy the predicate [f]. *)
+  val filter : t -> f:(elt -> bool) -> t
+
+  (** [filter_map t ~f] applies [f] to every [x] in [t]. The result contains every [y] for
+      which [f x] returns [Some y]. *)
+  val filter_map : t -> f:(elt -> elt option) -> t
+
+  (** [concat_map t ~f] is equivalent to [concat (map t ~f)]. *)
+  val concat_map : t -> f:(elt -> t) -> t
+
+  (** [partition_tf t ~f] returns a pair [t1, t2], where [t1] is all elements of [t] that
+      satisfy [f], and [t2] is all elements of [t] that do not satisfy [f]. The "tf"
+      suffix is mnemonic to remind readers that the result is (trues, falses). *)
+  val partition_tf : t -> f:(elt -> bool) -> t * t
+
+  (** [partition_map t ~f] partitions [t] according to [f]. *)
+  val partition_map : t -> f:(elt -> (elt, elt) Either0.t) -> t * t
+end
+
+module type S1_with_creators = sig
+  include S1
+
+  val of_list : 'a list -> 'a t
+  val of_array : 'a array -> 'a t
+
+  (** E.g., [append (of_list [1; 2]) (of_list [3; 4; 5])] is [of_list [1; 2; 3; 4; 5]] *)
+  val append : 'a t -> 'a t -> 'a t
+
+  (** Concatenates a nested container. The elements of the inner containers are
+      concatenated together in order to give the result. *)
+  val concat : 'a t t -> 'a t
+
+  (** [map f (of_list [a1; ...; an])] applies [f] to [a1], [a2], ..., [an], in order, and
+      builds a result equivalent to [of_list [f a1; ...; f an]]. *)
+  val map : 'a t -> f:('a -> 'b) -> 'b t
+
+  (** [filter t ~f] returns all the elements of [t] that satisfy the predicate [f]. *)
+  val filter : 'a t -> f:('a -> bool) -> 'a t
+
+  (** [filter_map t ~f] applies [f] to every [x] in [t]. The result contains every [y] for
+      which [f x] returns [Some y]. *)
+  val filter_map : 'a t -> f:('a -> 'b option) -> 'b t
+
+  (** [concat_map t ~f] is equivalent to [concat (map t ~f)]. *)
+  val concat_map : 'a t -> f:('a -> 'b t) -> 'b t
+
+  (** [partition_tf t ~f] returns a pair [t1, t2], where [t1] is all elements of [t] that
+      satisfy [f], and [t2] is all elements of [t] that do not satisfy [f]. The "tf"
+      suffix is mnemonic to remind readers that the result is (trues, falses). *)
+  val partition_tf : 'a t -> f:('a -> bool) -> 'a t * 'a t
+
+  (** [partition_map t ~f] partitions [t] according to [f]. *)
+  val partition_map : 'a t -> f:('a -> ('b, 'c) Either0.t) -> 'b t * 'c t
+end
+
+module type Generic_with_creators = sig
+  type (_, _, _) concat
+
+  include Generic
+
+  val of_list : 'a elt list -> ('a, _, _) t
+  val of_array : 'a elt array -> ('a, _, _) t
+  val append : ('a, 'p1, 'p2) t -> ('a, 'p1, 'p2) t -> ('a, 'p1, 'p2) t
+  val concat : (('a, 'p1, 'p2) t, 'p1, 'p2) concat -> ('a, 'p1, 'p2) t
+  val map : ('a, 'p1, 'p2) t -> f:('a elt -> 'b elt) -> ('b, 'p1, 'p2) t
+  val filter : ('a, 'p1, 'p2) t -> f:('a elt -> bool) -> ('a, 'p1, 'p2) t
+  val filter_map : ('a, 'p1, 'p2) t -> f:('a elt -> 'b elt option) -> ('b, 'p1, 'p2) t
+  val concat_map : ('a, 'p1, 'p2) t -> f:('a elt -> ('b, 'p1, 'p2) t) -> ('b, 'p1, 'p2) t
+
+  val partition_tf
+    :  ('a, 'p1, 'p2) t
+    -> f:('a elt -> bool)
+    -> ('a, 'p1, 'p2) t * ('a, 'p1, 'p2) t
+
+  val partition_map
+    :  ('a, 'p1, 'p2) t
+    -> f:('a elt -> ('b elt, 'c elt) Either0.t)
+    -> ('b, 'p1, 'p2) t * ('c, 'p1, 'p2) t
 end
 
 module type Make_gen_arg = sig
-  type 'a t
+  type ('a, 'phantom1, 'phantom2) t
   type 'a elt
 
-  val fold : 'a t -> init:'accum -> f:('accum -> 'a elt -> 'accum) -> 'accum
+  val fold
+    :  ('a, 'phantom1, 'phantom2) t
+    -> init:'acc
+    -> f:('acc -> 'a elt -> 'acc)
+    -> 'acc
 
   (** The [iter] argument to [Container.Make] specifies how to implement the
       container's [iter] function.  [`Define_using_fold] means to define [iter]
@@ -509,7 +572,10 @@ module type Make_gen_arg = sig
       efficient.  Several other functions returned by [Container.Make] are defined in
       terms of [iter], so passing in a more efficient [iter] will improve their efficiency
       as well. *)
-  val iter : [ `Define_using_fold | `Custom of 'a t -> f:('a elt -> unit) -> unit ]
+  val iter
+    : [ `Define_using_fold
+      | `Custom of ('a, 'phantom1, 'phantom2) t -> f:('a elt -> unit) -> unit
+      ]
 
   (** The [length] argument to [Container.Make] specifies how to implement the
       container's [length] function.  [`Define_using_fold] means to define
@@ -523,10 +589,14 @@ module type Make_gen_arg = sig
       efficient.  Several other functions returned by [Container.Make] are defined in
       terms of [length], so passing in a more efficient [length] will improve their
       efficiency as well. *)
-  val length : [ `Define_using_fold | `Custom of 'a t -> int ]
+  val length : [ `Define_using_fold | `Custom of ('a, 'phantom1, 'phantom2) t -> int ]
 end
 
-module type Make_arg = Make_gen_arg with type 'a elt := 'a Monad.Ident.t
+module type Make_arg = sig
+  type 'a t
+
+  include Make_gen_arg with type ('a, _, _) t := 'a t and type 'a elt := 'a
+end
 
 module type Make0_arg = sig
   module Elt : sig
@@ -537,28 +607,57 @@ module type Make0_arg = sig
 
   type t
 
-  include Make_gen_arg with type 'a t := t and type 'a elt := Elt.t
+  include Make_gen_arg with type ('a, _, _) t := t and type 'a elt := Elt.t
 end
 
-module type Container = sig
-  include module type of struct
-    include Export
+module type Make_common_with_creators_arg = sig
+  include Make_gen_arg
+
+  type (_, _, _) concat
+
+  val of_list : 'a elt list -> ('a, _, _) t
+  val of_array : 'a elt array -> ('a, _, _) t
+  val concat : (('a, _, _) t, _, _) concat -> ('a, _, _) t
+end
+
+module type Make_gen_with_creators_arg = sig
+  include Make_common_with_creators_arg
+
+  val concat_of_array : 'a array -> ('a, _, _) concat
+end
+
+module type Make_with_creators_arg = sig
+  type 'a t
+
+  include
+    Make_common_with_creators_arg
+      with type ('a, _, _) t := 'a t
+       and type 'a elt := 'a
+       and type ('a, _, _) concat := 'a t
+end
+
+module type Make0_with_creators_arg = sig
+  module Elt : sig
+    type t
+
+    val equal : t -> t -> bool
   end
 
-  module type S0 = S0
-  module type S0_phantom = S0_phantom
-  module type S1 = S1
-  module type S1_phantom_invariant = S1_phantom_invariant
-  module type S1_phantom = S1_phantom
-  module type Generic = Generic
-  module type Generic_phantom = Generic_phantom
-  module type Summable = Summable
+  type t
 
+  include
+    Make_common_with_creators_arg
+      with type ('a, _, _) t := t
+       and type 'a elt := Elt.t
+       and type ('a, _, _) concat := 'a list
+end
+
+module type Derived = sig
   (** Generic definitions of container operations in terms of [fold].
 
       E.g.: [iter ~fold t ~f = fold t ~init:() ~f:(fun () a -> f a)]. *)
 
-  type ('t, 'a, 'accum) fold = 't -> init:'accum -> f:('accum -> 'a -> 'accum) -> 'accum
+  type ('t, 'a, 'acc) fold = 't -> init:'acc -> f:('acc -> 'a -> 'acc) -> 'acc
   type ('t, 'a) iter = 't -> f:('a -> unit) -> unit
   type 't length = 't -> int
 
@@ -588,28 +687,48 @@ module type Container = sig
     -> 'sum
 
   val fold_result
-    :  fold:('t, 'a, 'b) fold
-    -> init:'b
-    -> f:('b -> 'a -> ('b, 'e) Result.t)
+    :  fold:('t, 'a, 'acc) fold
+    -> init:'acc
+    -> f:('acc -> 'a -> ('acc, 'e) Result.t)
     -> 't
-    -> ('b, 'e) Result.t
+    -> ('acc, 'e) Result.t
 
   val fold_until
-    :  fold:('t, 'a, 'b) fold
-    -> init:'b
-    -> f:('b -> 'a -> ('b, 'final) Continue_or_stop.t)
-    -> finish:('b -> 'final)
+    :  fold:('t, 'a, 'acc) fold
+    -> init:'acc
+    -> f:('acc -> 'a -> ('acc, 'final) Continue_or_stop.t)
+    -> finish:('acc -> 'final)
     -> 't
     -> 'final
 
   (** Generic definitions of container operations in terms of [iter] and [length]. *)
-  val is_empty : iter:('t, 'a) iter -> 't -> bool
 
+  val is_empty : iter:('t, 'a) iter -> 't -> bool
+  val mem : iter:('t, 'a) iter -> 't -> 'a -> equal:('a -> 'a -> bool) -> bool
   val exists : iter:('t, 'a) iter -> 't -> f:('a -> bool) -> bool
   val for_all : iter:('t, 'a) iter -> 't -> f:('a -> bool) -> bool
   val find : iter:('t, 'a) iter -> 't -> f:('a -> bool) -> 'a option
   val find_map : iter:('t, 'a) iter -> 't -> f:('a -> 'b option) -> 'b option
   val to_array : length:'t length -> iter:('t, 'a) iter -> 't -> 'a array
+end
+
+module type Container = sig
+  include module type of struct
+    include Export
+  end
+
+  module type S0 = S0
+  module type S0_phantom = S0_phantom
+  module type S0_with_creators = S0_with_creators
+  module type S1 = S1
+  module type S1_phantom = S1_phantom
+  module type S1_with_creators = S1_with_creators
+  module type Derived = Derived
+  module type Generic = Generic
+  module type Generic_with_creators = Generic_with_creators
+  module type Summable = Summable
+
+  include Derived
 
   (** The idiom for using [Container.Make] is to bind the resulting module and to
       explicitly import each of the functions that one wants:
@@ -638,5 +757,19 @@ module type Container = sig
   module Make0 (T : Make0_arg) : S0 with type t := T.t and type elt := T.Elt.t
 
   module Make_gen (T : Make_gen_arg) :
-    Generic with type 'a t := 'a T.t and type 'a elt := 'a T.elt
+    Generic
+      with type ('a, 'phantom1, 'phantom2) t := ('a, 'phantom1, 'phantom2) T.t
+       and type 'a elt := 'a T.elt
+
+  module Make_with_creators (T : Make_with_creators_arg) :
+    S1_with_creators with type 'a t := 'a T.t
+
+  module Make0_with_creators (T : Make0_with_creators_arg) :
+    S0_with_creators with type t := T.t and type elt := T.Elt.t
+
+  module Make_gen_with_creators (T : Make_gen_with_creators_arg) :
+    Generic_with_creators
+      with type ('a, 'phantom1, 'phantom2) t := ('a, 'phantom1, 'phantom2) T.t
+       and type 'a elt := 'a T.elt
+       and type ('a, 'phantom1, 'phantom2) concat := ('a, 'phantom1, 'phantom2) T.concat
 end
