@@ -19,12 +19,7 @@ module Key = struct
   type 'a t = (module S with type t = 'a)
 end
 
-(** @canonical Base.Hashtbl.Merge_into_action *)
-module Merge_into_action = struct
-  type 'a t =
-    | Remove
-    | Set_to of 'a
-end
+module Merge_into_action = Dictionary_mutable.Merge_into_action
 
 module type Accessors = sig
   (** {2 Accessors} *)
@@ -32,13 +27,20 @@ module type Accessors = sig
   type ('a, 'b) t
   type 'a key
 
+  (** @inline *)
+  include
+    Dictionary_mutable.Accessors
+      with type 'key key := 'key key
+       and type ('key, 'data, _) t := ('key, 'data) t
+       and type ('fn, _, _, _) accessor := 'fn
+
   val sexp_of_key : ('a, _) t -> 'a key -> Sexp.t
   val clear : (_, _) t -> unit
   val copy : ('a, 'b) t -> ('a, 'b) t
 
   (** Attempting to modify ([set], [remove], etc.) the hashtable during iteration ([fold],
       [iter], [iter_keys], [iteri]) will raise an exception. *)
-  val fold : ('a, 'b) t -> init:'c -> f:(key:'a key -> data:'b -> 'c -> 'c) -> 'c
+  val fold : ('a, 'b) t -> init:'acc -> f:(key:'a key -> data:'b -> 'acc -> 'acc) -> 'acc
 
   val iter_keys : ('a, _) t -> f:('a key -> unit) -> unit
   val iter : (_, 'b) t -> f:('b -> unit) -> unit
@@ -64,12 +66,45 @@ module type Accessors = sig
   val counti : ('a, 'b) t -> f:(key:'a key -> data:'b -> bool) -> int
   val count : (_, 'b) t -> f:('b -> bool) -> int
   val length : (_, _) t -> int
+  val capacity : _ t -> int
   val is_empty : (_, _) t -> bool
   val mem : ('a, _) t -> 'a key -> bool
   val remove : ('a, _) t -> 'a key -> unit
+
+  (** Choose an arbitrary key/value pair of a hash table. Returns [None] if [t] is empty.
+
+      The choice is deterministic. Calling [choose] multiple times on the same table
+      returns the same key/value pair, so long as the table is not mutated in between.
+      Beyond determinism, no guarantees are made about how the choice is made. Expect
+      bias toward certain hash values.
+
+      This hash bias can lead to degenerate performance in some cases, such as clearing
+      a hash table using repeated [choose] and [remove]. At each iteration, finding the
+      next element may have to scan farther from its initial hash value. *)
   val choose : ('a, 'b) t -> ('a key * 'b) option
+
+  (** Like [choose]. Raises if [t] is empty. *)
   val choose_exn : ('a, 'b) t -> 'a key * 'b
 
+  (** Chooses a random key/value pair of a hash table. Returns [None] if [t] is empty.
+
+      The choice is distributed uniformly across hash values, rather than across keys
+      themselves. As a consequence, the closer the keys are to evenly spaced out in the
+      table, the closer this function will be to a uniform choice of keys.
+
+      This function may be preferable to [choose] when nondeterministic choice is
+      acceptable, and bias toward certain hash values is undesirable. *)
+  val choose_randomly
+    :  ?random_state:Random.State.t (** default: [Random.State.default] *)
+    -> ('a, 'b) t
+    -> ('a key * 'b) option
+
+  (** Like [choose_randomly]. Raises if [t] is empty. *)
+  val choose_randomly_exn
+    :  ?random_state:Random.State.t (** default: [Random.State.default] *)
+    -> ('a, 'b) t
+    -> 'a key * 'b
+
   (** Sets the given [key] to [data]. *)
   val set : ('a, 'b) t -> key:'a key -> data:'b -> unit
 
@@ -94,7 +129,7 @@ module type Accessors = sig
 
       {v
       let h = Hashtbl.of_alist_exn (module Int) [(1, 4); (5, 6)] in
-      let h' = Hashtbl.map h ~f:(fun x -> x * 2) in
+      let h' = Hashtbl.map h ~f:(local_ (fun x -> x * 2)) in
       Hashtbl.to_alist h';;
       - : (int * int) list = [(5, 12); (1, 8)]
       v} *)
@@ -111,7 +146,7 @@ module type Accessors = sig
 
       {v
       let h = Hashtbl.of_alist_exn (module Int) [(1, 4); (5, 6)] in
-      Hashtbl.filter_map h ~f:(fun x -> if x > 5 then Some x else None)
+      Hashtbl.filter_map h ~f:(local_ (fun x -> if x > 5 then Some x else None))
       |> Hashtbl.to_alist;;
       - : (int * int) list = [(5, 6)]
       v} *)
@@ -155,7 +190,7 @@ module type Accessors = sig
       binding exists. *)
   val find : ('a, 'b) t -> 'a key -> 'b option
 
-  (** [find_exn t k] returns the current binding of [k] in [t], or raises [Caml.Not_found]
+  (** [find_exn t k] returns the current binding of [k] in [t], or raises [Stdlib.Not_found]
       or [Not_found_s] if no such binding exists. *)
   val find_exn : ('a, 'b) t -> 'a key -> 'b
 
@@ -258,13 +293,12 @@ module type Accessors = sig
     -> f:(key:'k key -> [ `Left of 'a | `Right of 'b | `Both of 'a * 'b ] -> 'c option)
     -> ('k, 'c) t
 
-
   (** Every [key] in [src] will be removed or set in [dst] according to the return value
       of [f]. *)
   val merge_into
     :  src:('k, 'a) t
     -> dst:('k, 'b) t
-    -> f:(key:'k key -> 'a -> 'b option -> 'b Merge_into_action.t)
+    -> f:(key:'k key -> 'a -> 'b option -> 'b Dictionary_mutable.Merge_into_action.t)
     -> unit
 
   (** Returns the list of all keys for given hashtable. *)
@@ -300,7 +334,6 @@ module type Accessors = sig
   (** Returns the list of all (key, data) pairs for given hashtable. *)
   val to_alist : ('a, 'b) t -> ('a key * 'b) list
 
-
   (** [remove_if_zero]'s default is [false]. *)
   val incr : ?by:int -> ?remove_if_zero:bool -> ('a, int) t -> 'a key -> unit
 
@@ -341,20 +374,26 @@ module type Creators_generic = sig
   type 'a key
   type ('key, 'data, 'z) create_options
 
-  val create : ('a key, 'b, unit -> ('a, 'b) t) create_options
+  (** @inline *)
+  include
+    Dictionary_mutable.Creators
+      with type 'key key := 'key key
+       and type ('key, 'data, _) t := ('key, 'data) t
+       and type ('fn, 'key, 'data, _) creator := ('key key, 'data, 'fn) create_options
 
+  val create : ('a key, 'b, unit -> ('a, 'b) t) create_options
 
   val of_alist
     : ( 'a key
       , 'b
       , ('a key * 'b) list -> [ `Ok of ('a, 'b) t | `Duplicate_key of 'a key ] )
-        create_options
+      create_options
 
   val of_alist_report_all_dups
     : ( 'a key
       , 'b
       , ('a key * 'b) list -> [ `Ok of ('a, 'b) t | `Duplicate_keys of 'a key list ] )
-        create_options
+      create_options
 
   val of_alist_or_error
     : ('a key, 'b, ('a key * 'b) list -> ('a, 'b) t Or_error.t) create_options
@@ -364,7 +403,6 @@ module type Creators_generic = sig
   val of_alist_multi
     : ('a key, 'b list, ('a key * 'b) list -> ('a, 'b list) t) create_options
 
-
   (** {[ create_mapped get_key get_data [x1,...,xn]
          = of_alist [get_key x1, get_data x1; ...; get_key xn, get_data xn] ]} *)
   val create_mapped
@@ -374,8 +412,7 @@ module type Creators_generic = sig
         -> get_data:('r -> 'b)
         -> 'r list
         -> [ `Ok of ('a, 'b) t | `Duplicate_keys of 'a key list ] )
-        create_options
-
+      create_options
 
   (** {[ create_with_key ~get_key [x1,...,xn]
          = of_alist [get_key x1, x1; ...; get_key xn, xn] ]} *)
@@ -385,18 +422,17 @@ module type Creators_generic = sig
       , get_key:('r -> 'a key)
         -> 'r list
         -> [ `Ok of ('a, 'r) t | `Duplicate_keys of 'a key list ] )
-        create_options
+      create_options
 
   val create_with_key_or_error
     : ( 'a key
       , 'r
       , get_key:('r -> 'a key) -> 'r list -> ('a, 'r) t Or_error.t )
-        create_options
+      create_options
 
   val create_with_key_exn
     : ('a key, 'r, get_key:('r -> 'a key) -> 'r list -> ('a, 'r) t) create_options
 
-
   val group
     : ( 'a key
       , 'b
@@ -405,7 +441,7 @@ module type Creators_generic = sig
         -> combine:('b -> 'b -> 'b)
         -> 'r list
         -> ('a, 'b) t )
-        create_options
+      create_options
 end
 
 module type Creators = sig
@@ -505,8 +541,8 @@ module type Creators = sig
       {v
         let h =
           Hashtbl.create_mapped (module Int)
-            ~get_key:(fun x -> x)
-            ~get_data:(fun x -> x + 1)
+            ~get_key:(local_ (fun x -> x))
+            ~get_data:(local_ (fun x -> x + 1))
            [1; 2; 3];;
         val h : [ `Duplicate_keys of int list | `Ok of (int, int) Hashtbl.t ] = `Ok <abstr>
 
@@ -563,9 +599,9 @@ module type Creators = sig
 
       {v
          Hashtbl.group (module Int)
-           ~get_key:(fun x -> x / 2)
-           ~get_data:(fun x -> x)
-           ~combine:(fun x y -> x * y)
+           ~get_key:(local_ (fun x -> x / 2))
+           ~get_data:(local_ (fun x -> x))
+           ~combine:(local_ (fun x y -> x * y))
             [ 1; 2; 3; 4]
          |> Hashtbl.to_alist;;
          - : (int * int) list = [(2, 4); (1, 6); (0, 1)]
@@ -585,7 +621,7 @@ module type S_without_submodules = sig
   val hash : 'a -> int
   val hash_param : int -> int -> 'a -> int
 
-  type ('a, 'b) t
+  type (!'a, !'b) t
 
   (** We provide a [sexp_of_t] but not a [t_of_sexp] for this type because one needs to be
       explicit about the hash and comparison functions used when creating a hashtable.
@@ -598,7 +634,6 @@ module type S_without_submodules = sig
   include Accessors with type ('a, 'b) t := ('a, 'b) t with type 'a key = 'a
   (** @inline *)
 
-
   include Multi with type ('a, 'b) t := ('a, 'b) t with type 'a key := 'a key
   (** @inline *)
 
@@ -625,10 +660,10 @@ module type S_poly = sig
 
   include
     Creators_generic
-    with type ('a, 'b) t := ('a, 'b) t
-    with type 'a key = 'a
-    with type ('key, 'data, 'z) create_options :=
-      ('key, 'data, 'z) create_options_without_first_class_module
+      with type ('a, 'b) t := ('a, 'b) t
+      with type 'a key = 'a
+      with type ('key, 'data, 'z) create_options :=
+        ('key, 'data, 'z) create_options_without_first_class_module
 
   include Accessors with type ('a, 'b) t := ('a, 'b) t with type 'a key := 'a key
   include Multi with type ('a, 'b) t := ('a, 'b) t with type 'a key := 'a key
@@ -780,20 +815,20 @@ module type Hashtbl = sig
   type nonrec ('key, 'data, 'z) create_options = ('key, 'data, 'z) create_options
 
   module Creators (Key : sig
-      type 'a t
+    type 'a t
 
-      val hashable : 'a t Hashable.t
-    end) : sig
+    val hashable : 'a t Hashable.t
+  end) : sig
     type ('a, 'b) t_ = ('a Key.t, 'b) t
 
     val t_of_sexp : (Sexp.t -> 'a Key.t) -> (Sexp.t -> 'b) -> Sexp.t -> ('a, 'b) t_
 
     include
       Creators_generic
-      with type ('a, 'b) t := ('a, 'b) t_
-      with type 'a key := 'a Key.t
-      with type ('key, 'data, 'a) create_options :=
-        ('key, 'data, 'a) create_options_without_first_class_module
+        with type ('a, 'b) t := ('a, 'b) t_
+        with type 'a key := 'a Key.t
+        with type ('key, 'data, 'a) create_options :=
+          ('key, 'data, 'a) create_options_without_first_class_module
   end
 
   module Poly : S_poly with type ('a, 'b) t = ('a, 'b) t
