@@ -3,7 +3,9 @@ include Int_intf
 include Int0
 
 module T = struct
-  type t = int [@@deriving_inline hash, sexp, sexp_grammar]
+  type t = int [@@deriving_inline globalize, hash, sexp, sexp_grammar]
+
+  let (globalize : t -> t) = (globalize_int : t -> t)
 
   let (hash_fold_t : Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state) =
     hash_fold_int
@@ -33,13 +35,13 @@ end
 let num_bits = Int_conversions.num_bits_int
 let float_lower_bound = Float0.lower_bound_for_int num_bits
 let float_upper_bound = Float0.upper_bound_for_int num_bits
-let to_float = Caml.float_of_int
-let of_float_unchecked = Caml.int_of_float
+let to_float = Stdlib.float_of_int
+let of_float_unchecked = Stdlib.int_of_float
 
 let of_float f =
   if Float_replace_polymorphic_compare.( >= ) f float_lower_bound
-  && Float_replace_polymorphic_compare.( <= ) f float_upper_bound
-  then Caml.int_of_float f
+     && Float_replace_polymorphic_compare.( <= ) f float_upper_bound
+  then Stdlib.int_of_float f
   else
     Printf.invalid_argf
       "Int.of_float: argument (%f) is out of range or NaN"
@@ -55,45 +57,46 @@ include T
 include Comparator.Make (T)
 
 include Comparable.With_zero (struct
-    include T
+  include T
 
-    let zero = zero
-  end)
+  let zero = zero
+end)
 
 module Conv = Int_conversions
-include Conv.Make (T)
+include Int_string_conversions.Make (T)
 
-include Conv.Make_hex (struct
-    open Int_replace_polymorphic_compare
+include Int_string_conversions.Make_hex (struct
+  open Int_replace_polymorphic_compare
 
-    type t = int [@@deriving_inline compare, hash]
+  type t = int [@@deriving_inline compare ~localize, hash]
 
-    let compare = (compare_int : t -> t -> int)
+  let compare__local = (compare_int__local : t -> t -> int)
+  let compare = (fun a b -> compare__local a b : t -> t -> int)
 
-    let (hash_fold_t : Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state) =
-      hash_fold_int
+  let (hash_fold_t : Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state) =
+    hash_fold_int
 
-    and (hash : t -> Ppx_hash_lib.Std.Hash.hash_value) =
-      let func = hash_int in
-      fun x -> func x
-    ;;
+  and (hash : t -> Ppx_hash_lib.Std.Hash.hash_value) =
+    let func = hash_int in
+    fun x -> func x
+  ;;
 
-    [@@@end]
+  [@@@end]
 
-    let zero = zero
-    let neg = ( ~- )
-    let ( < ) = ( < )
-    let to_string i = Printf.sprintf "%x" i
-    let of_string s = Caml.Scanf.sscanf s "%x" Fn.id
-    let module_name = "Base.Int.Hex"
-  end)
+  let zero = zero
+  let neg = ( ~- )
+  let ( < ) = ( < )
+  let to_string i = Printf.sprintf "%x" i
+  let of_string s = Stdlib.Scanf.sscanf s "%x" Fn.id
+  let module_name = "Base.Int.Hex"
+end)
 
 include Pretty_printer.Register (struct
-    type nonrec t = t
+  type nonrec t = t
 
-    let to_string = to_string
-    let module_name = "Base.Int"
-  end)
+  let to_string = to_string
+  let module_name = "Base.Int"
+end)
 
 (* Open replace_polymorphic_compare after including functor instantiations so
    they do not shadow its definitions. This is here so that efficient versions
@@ -102,7 +105,7 @@ open! Int_replace_polymorphic_compare
 
 let invariant (_ : t) = ()
 let between t ~low ~high = low <= t && t <= high
-let clamp_unchecked t ~min ~max = if t < min then min else if t <= max then t else max
+let clamp_unchecked t ~min:min_ ~max:max_ = min t max_ |> max min_
 
 let clamp_exn t ~min ~max =
   assert (min <= max);
@@ -119,10 +122,10 @@ let clamp t ~min ~max =
   else Ok (clamp_unchecked t ~min ~max)
 ;;
 
-external to_int32_trunc : t -> int32 = "%int32_of_int"
-external of_int32_trunc : int32 -> t = "%int32_to_int"
-external of_int64_trunc : int64 -> t = "%int64_to_int"
-external of_nativeint_trunc : nativeint -> t = "%nativeint_to_int"
+external to_int32_trunc : (t[@local_opt]) -> (int32[@local_opt]) = "%int32_of_int"
+external of_int32_trunc : (int32[@local_opt]) -> t = "%int32_to_int"
+external of_int64_trunc : (int64[@local_opt]) -> t = "%int64_to_int"
+external of_nativeint_trunc : (nativeint[@local_opt]) -> t = "%nativeint_to_int"
 
 let pred i = i - 1
 let succ i = i + 1
@@ -130,8 +133,8 @@ let to_int i = i
 let to_int_exn = to_int
 let of_int i = i
 let of_int_exn = of_int
-let max_value = Caml.max_int
-let min_value = Caml.min_int
+let max_value = Stdlib.max_int
+let min_value = Stdlib.min_int
 let max_value_30_bits = 0x3FFF_FFFF
 let of_int32 = Conv.int32_to_int
 let of_int32_exn = Conv.int32_to_int_exn
@@ -148,8 +151,8 @@ let abs x = abs x
 
 (* note that rem is not same as % *)
 let rem a b = a mod b
-let incr = Caml.incr
-let decr = Caml.decr
+let incr = Stdlib.incr
+let decr = Stdlib.decr
 let shift_right a b = a asr b
 let shift_right_logical a b = a lsr b
 let shift_left a b = a lsl b
@@ -169,7 +172,6 @@ module Pow2 = struct
     Printf.invalid_argf "argument must be strictly positive" ()
   ;;
 
-
   (** "ceiling power of 2" - Least power of 2 greater than or equal to x. *)
   let ceil_pow2 x =
     if x <= 0 then non_positive_argument ();
@@ -208,16 +210,16 @@ module Pow2 = struct
   external clz
     :  (* Note that we pass the tagged int here. See int_math_stubs.c for details on why
           this is correct. *)
-    int
+       int
     -> (int[@untagged])
     = "Base_int_math_int_clz" "Base_int_math_int_clz_untagged"
-  [@@noalloc]
+    [@@noalloc]
 
   external ctz
     :  (int[@untagged])
     -> (int[@untagged])
     = "Base_int_math_int_ctz" "Base_int_math_int_ctz_untagged"
-  [@@noalloc]
+    [@@noalloc]
 
   (** Hacker's Delight Second Edition p106 *)
   let floor_log2 i =
@@ -238,12 +240,39 @@ include Pow2
 let sign = Sign.of_int
 let popcount = Popcount.int_popcount
 
+include Int_string_conversions.Make_binary (struct
+  type t = int [@@deriving_inline compare ~localize, equal ~localize, hash]
+
+  let compare__local = (compare_int__local : t -> t -> int)
+  let compare = (fun a b -> compare__local a b : t -> t -> int)
+  let equal__local = (equal_int__local : t -> t -> bool)
+  let equal = (fun a b -> equal__local a b : t -> t -> bool)
+
+  let (hash_fold_t : Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state) =
+    hash_fold_int
+
+  and (hash : t -> Ppx_hash_lib.Std.Hash.hash_value) =
+    let func = hash_int in
+    fun x -> func x
+  ;;
+
+  [@@@end]
+
+  let ( land ) = ( land )
+  let ( lsr ) = ( lsr )
+  let clz = clz
+  let num_bits = num_bits
+  let one = one
+  let to_int_exn = to_int_exn
+  let zero = zero
+end)
+
 module Pre_O = struct
-  external ( + ) : int -> int -> int = "%addint"
-  external ( - ) : int -> int -> int = "%subint"
-  external ( * ) : int -> int -> int = "%mulint"
-  external ( / ) : int -> int -> int = "%divint"
-  external ( ~- ) : int -> int = "%negint"
+  external ( + ) : (t[@local_opt]) -> (t[@local_opt]) -> t = "%addint"
+  external ( - ) : (t[@local_opt]) -> (t[@local_opt]) -> t = "%subint"
+  external ( * ) : (t[@local_opt]) -> (t[@local_opt]) -> t = "%mulint"
+  external ( / ) : (t[@local_opt]) -> (t[@local_opt]) -> t = "%divint"
+  external ( ~- ) : (t[@local_opt]) -> t = "%negint"
 
   let ( ** ) = ( ** )
 
@@ -251,7 +280,7 @@ module Pre_O = struct
 
   let abs = abs
 
-  external neg : t -> t = "%negint"
+  external neg : (t[@local_opt]) -> t = "%negint"
 
   let zero = zero
   let of_int_exn = of_int_exn
@@ -261,20 +290,20 @@ module O = struct
   include Pre_O
 
   module F = Int_math.Make (struct
-      type nonrec t = t
+    type nonrec t = t
 
-      include Pre_O
+    include Pre_O
 
-      let rem = rem
-      let to_float = to_float
-      let of_float = of_float
-      let of_string = T.of_string
-      let to_string = T.to_string
-    end)
+    let rem = rem
+    let to_float = to_float
+    let of_float = of_float
+    let of_string = T.of_string
+    let to_string = T.to_string
+  end)
 
   include F
 
-  external bswap16 : int -> int = "%bswap16"
+  external bswap16 : (int[@local_opt]) -> int = "%bswap16"
 
   (* These inlined versions of (%), (/%), and (//) perform better than their functorized
      counterparts in [F] (see benchmarks below).
@@ -311,15 +340,15 @@ module O = struct
 
   let ( // ) x y = to_float x /. to_float y
 
-  external ( land ) : int -> int -> int = "%andint"
-  external ( lor ) : int -> int -> int = "%orint"
-  external ( lxor ) : int -> int -> int = "%xorint"
+  external ( land ) : (int[@local_opt]) -> (int[@local_opt]) -> int = "%andint"
+  external ( lor ) : (int[@local_opt]) -> (int[@local_opt]) -> int = "%orint"
+  external ( lxor ) : (int[@local_opt]) -> (int[@local_opt]) -> int = "%xorint"
 
   let lnot = lnot
 
-  external ( lsl ) : int -> int -> int = "%lslint"
-  external ( lsr ) : int -> int -> int = "%lsrint"
-  external ( asr ) : int -> int -> int = "%asrint"
+  external ( lsl ) : (int[@local_opt]) -> (int[@local_opt]) -> int = "%lslint"
+  external ( lsr ) : (int[@local_opt]) -> (int[@local_opt]) -> int = "%lsrint"
+  external ( asr ) : (int[@local_opt]) -> (int[@local_opt]) -> int = "%asrint"
 end
 
 include O
