@@ -4,18 +4,25 @@ include Linked_queue0
 let enqueue t x = Linked_queue0.push x t
 let dequeue t = if is_empty t then None else Some (Linked_queue0.pop t)
 let dequeue_exn = Linked_queue0.pop
+let dequeue_and_ignore_exn (type elt) (t : elt t) = ignore (dequeue_exn t : elt)
 let peek t = if is_empty t then None else Some (Linked_queue0.peek t)
 let peek_exn = Linked_queue0.peek
 
-module C = Indexed_container.Make (struct
-    type nonrec 'a t = 'a t
+let drain t ~f ~while_ =
+  while (not (is_empty t)) && while_ (peek_exn t) do
+    f (dequeue_exn t)
+  done
+;;
 
-    let fold = fold
-    let iter = `Custom iter
-    let length = `Custom length
-    let foldi = `Define_using_fold
-    let iteri = `Define_using_fold
-  end)
+module C = Indexed_container.Make (struct
+  type nonrec 'a t = 'a t
+
+  let fold = fold
+  let iter = `Custom iter
+  let length = `Custom length
+  let foldi = `Define_using_fold
+  let iteri = `Define_using_fold
+end)
 
 let count = C.count
 let exists = C.exists
@@ -141,6 +148,12 @@ let to_array t =
 let t_of_sexp a_of_sexp sexp = of_list (list_of_sexp a_of_sexp sexp)
 let sexp_of_t sexp_of_a t = sexp_of_list sexp_of_a (to_list t)
 
+let t_sexp_grammar (type a) (grammar : a Sexplib0.Sexp_grammar.t)
+  : a t Sexplib0.Sexp_grammar.t
+  =
+  Sexplib0.Sexp_grammar.coerce (List.t_sexp_grammar grammar)
+;;
+
 let singleton a =
   let t = create () in
   enqueue t a;
