@@ -24,7 +24,7 @@ module type S = sig
   val of_array : 'a array -> 'a t
 
   (** [init n ~f] is equivalent to [of_list (List.init n ~f)]. *)
-  val init : int -> f:((int -> 'a)[@local]) -> 'a t
+  val init : int -> f:(int -> 'a) -> 'a t
 
   (** [enqueue t a] adds [a] to the end of [t].*)
   val enqueue : 'a t -> 'a -> unit
@@ -41,6 +41,13 @@ module type S = sig
       is empty. *)
   val dequeue_and_ignore_exn : 'a t -> unit
 
+  (** [drain t ~f ~while_] repeatedly calls [while_] on the head of [t], and if it returns
+      true then dequeues it and calls [f] on it. It stops when [t] is empty or [while_]
+      returns false. A common use case is tracking the sum of data in a recent time
+      interval: [t] contains timestamped data, [while_] checks for elements with an old
+      timestamp, and [f] subtracts data from the sum. *)
+  val drain : 'a t -> f:('a -> unit) -> while_:('a -> bool) -> unit
+
   (** [peek t] returns but does not remove the front element of [t], if any. *)
   val peek : 'a t -> 'a option
 
@@ -52,31 +59,31 @@ module type S = sig
   (** [copy t] returns a copy of [t]. *)
   val copy : 'a t -> 'a t
 
-  val map : 'a t -> f:(('a -> 'b)[@local]) -> 'b t
-  val mapi : 'a t -> f:((int -> 'a -> 'b)[@local]) -> 'b t
+  val map : 'a t -> f:('a -> 'b) -> 'b t
+  val mapi : 'a t -> f:(int -> 'a -> 'b) -> 'b t
 
   (** Creates a new queue with elements equal to [List.concat_map ~f (to_list t)]. *)
-  val concat_map : 'a t -> f:(('a -> 'b list)[@local]) -> 'b t
+  val concat_map : 'a t -> f:('a -> 'b list) -> 'b t
 
-  val concat_mapi : 'a t -> f:((int -> 'a -> 'b list)[@local]) -> 'b t
+  val concat_mapi : 'a t -> f:(int -> 'a -> 'b list) -> 'b t
 
   (** [filter_map] creates a new queue with elements equal to [List.filter_map ~f (to_list
       t)]. *)
-  val filter_map : 'a t -> f:(('a -> 'b option)[@local]) -> 'b t
+  val filter_map : 'a t -> f:('a -> 'b option) -> 'b t
 
-  val filter_mapi : 'a t -> f:((int -> 'a -> 'b option)[@local]) -> 'b t
+  val filter_mapi : 'a t -> f:(int -> 'a -> 'b option) -> 'b t
 
   (** [filter] is like [filter_map], except with [List.filter]. *)
-  val filter : 'a t -> f:(('a -> bool)[@local]) -> 'a t
+  val filter : 'a t -> f:('a -> bool) -> 'a t
 
-  val filteri : 'a t -> f:((int -> 'a -> bool)[@local]) -> 'a t
+  val filteri : 'a t -> f:(int -> 'a -> bool) -> 'a t
 
   (** [filter_inplace t ~f] removes all elements of [t] that don't satisfy [f].  If [f]
       raises, [t] is unchanged.  This is inplace in that it modifies [t]; however, it uses
       space linear in the final length of [t]. *)
-  val filter_inplace : 'a t -> f:(('a -> bool)[@local]) -> unit
+  val filter_inplace : 'a t -> f:('a -> bool) -> unit
 
-  val filteri_inplace : 'a t -> f:((int -> 'a -> bool)[@local]) -> unit
+  val filteri_inplace : 'a t -> f:(int -> 'a -> bool) -> unit
 end
 
 module type Queue = sig
@@ -94,14 +101,18 @@ module type Queue = sig
 
   module type S = S
 
-  type 'a t [@@deriving_inline compare]
+  type 'a t [@@deriving_inline compare ~localize, globalize]
 
   include Ppx_compare_lib.Comparable.S1 with type 'a t := 'a t
+  include Ppx_compare_lib.Comparable.S_local1 with type 'a t := 'a t
+
+  val globalize : ('a -> 'a) -> 'a t -> 'a t
 
   [@@@end]
 
   include S with type 'a t := 'a t
   include Equal.S1 with type 'a t := 'a t
+  include Ppx_compare_lib.Equal.S_local1 with type 'a t := 'a t
   include Invariant.S1 with type 'a t := 'a t
 
   (** Create an empty queue. *)
@@ -112,6 +123,20 @@ module type Queue = sig
 
   val last_exn : 'a t -> 'a
 
+  (** Add an element to the front of the queue, as opposed to [enqueue] which adds to the
+      back of the queue. *)
+  val enqueue_front : 'a t -> 'a -> unit
+
+  (** [dequeue_back t] removes and returns the back element of [t], if any. *)
+  val dequeue_back : 'a t -> 'a option
+
+  val dequeue_back_exn : 'a t -> 'a
+
+  (** [peek_back t] returns but does not remove the back element of [t], if any. *)
+  val peek_back : 'a t -> 'a option
+
+  val peek_back_exn : 'a t -> 'a
+
   (** Transfers up to [len] elements from the front of [src] to the end of [dst], removing
       them from [src].  It is an error if [len < 0].
 
@@ -138,4 +163,25 @@ module type Queue = sig
       [c < capacity t]. *)
   val set_capacity : _ t -> int -> unit
 
+  (** Use [Iteration] to implement iteration functions that guard against mutation. *)
+  module Iteration : sig
+    type 'a queue := 'a t
+
+    (** A token representing state from the beginning of an iteration. *)
+    type t [@@immediate]
+
+    (** Capture state at the start of iteration. *)
+    val start : _ queue -> t
+
+    (** [assert_no_mutation_since_start t queue] raises if any mutation has happened to
+        [queue] since [t] was created by [start queue]. Results are unspecified if you
+        call [assert_no_mutation_since_start] with a different queue from the one passed
+        to [start].
+
+        Call [assert_no_mutation_since_start] after each step of an iteration loop, before
+        checking if the queue is empty or advancing to the next element. This ensures
+        these read operations are consistent with the queue's state at the start of
+        iteration. *)
+    val assert_no_mutation_since_start : t -> _ queue -> unit
+  end
 end
