@@ -3,11 +3,16 @@
 
 open! Import
 
-type 'a t = 'a Caml.ref = { mutable contents : 'a }
-[@@deriving_inline compare, equal, sexp, sexp_grammar]
+type 'a t = 'a Stdlib.ref = { mutable contents : 'a }
+[@@deriving_inline compare ~localize, equal ~localize, globalize, sexp, sexp_grammar]
 
 include Ppx_compare_lib.Comparable.S1 with type 'a t := 'a t
+include Ppx_compare_lib.Comparable.S_local1 with type 'a t := 'a t
 include Ppx_compare_lib.Equal.S1 with type 'a t := 'a t
+include Ppx_compare_lib.Equal.S_local1 with type 'a t := 'a t
+
+val globalize : ('a -> 'a) -> 'a t -> 'a t
+
 include Sexplib0.Sexpable.S1 with type 'a t := 'a t
 
 val t_sexp_grammar : 'a Sexplib0.Sexp_grammar.t -> 'a t Sexplib0.Sexp_grammar.t
@@ -16,9 +21,9 @@ val t_sexp_grammar : 'a Sexplib0.Sexp_grammar.t -> 'a t Sexplib0.Sexp_grammar.t
 
 (*_ defined as externals to avoid breaking the inliner *)
 
-external create : 'a -> 'a t = "%makemutable"
-external ( ! ) : 'a t -> 'a = "%field0"
-external ( := ) : 'a t -> 'a -> unit = "%setfield0"
+external create : 'a -> ('a t[@local_opt]) = "%makemutable"
+external ( ! ) : ('a t[@local_opt]) -> 'a = "%field0"
+external ( := ) : ('a t[@local_opt]) -> 'a -> unit = "%setfield0"
 
 (** [swap t1 t2] swaps the values in [t1] and [t2]. *)
 val swap : 'a t -> 'a t -> unit
