@@ -11,6 +11,8 @@ let () =
     Printf.fprintf
       oc
       {|
+type t = Stdlib.Random.State.t Stdlib.Domain.DLS.key
+
 module Repr = struct
   open Stdlib.Bigarray
 
@@ -19,18 +21,25 @@ module Repr = struct
   let of_state : Stdlib.Random.State.t -> t = Stdlib.Obj.magic
 end
 
-let assign dst src =
-  let dst = Repr.of_state (Lazy.force dst) in
-  let src = Repr.of_state (Lazy.force src) in
+let assign t state =
+  let dst = Repr.of_state (Stdlib.Domain.DLS.get t) in
+  let src = Repr.of_state state in
   Stdlib.Bigarray.Array1.blit src dst
+;;
 
-let make_default default =
-  let split_from_parent v =
-    Stdlib.Lazy.map_val Stdlib.Random.State.split v
-  in
-  Stdlib.Domain.DLS.new_key ~split_from_parent (fun () -> default)
+let make state =
+  let split_from_parent v = Stdlib.Random.State.split v in
+  let t = Stdlib.Domain.DLS.new_key ~split_from_parent (fun () -> state) in
+  Stdlib.Domain.DLS.get t |> ignore;
+  t
+;;
 
-let get_state random_key = Stdlib.Domain.DLS.get random_key
+let make_lazy ~f =
+  let split_from_parent v = Stdlib.Random.State.split v in
+  Stdlib.Domain.DLS.new_key ~split_from_parent f
+;;
+
+let[@inline always] get_state t = Stdlib.Domain.DLS.get t
 |}
   else
     Printf.fprintf
@@ -38,6 +47,8 @@ let get_state random_key = Stdlib.Domain.DLS.get random_key
       {|
 module Array = Array0
 
+type t = Stdlib.Random.State.t Lazy.t
+
 module Repr = struct
   type t =
     { st : int array
@@ -47,15 +58,17 @@ module Repr = struct
   let of_state : Stdlib.Random.State.t -> t = Stdlib.Obj.magic
 end
 
-let assign t1 t2 =
-  let t1 = Repr.of_state (Lazy.force t1) in
-  let t2 = Repr.of_state (Lazy.force t2) in
+let assign t state =
+  let t1 = Repr.of_state (Lazy.force t) in
+  let t2 = Repr.of_state state in
   Array.blit ~src:t2.st ~src_pos:0 ~dst:t1.st ~dst_pos:0 ~len:(Array.length t1.st);
   t1.idx <- t2.idx
 
-let make_default default = default
+let make state = Lazy.from_val state
 
-let[@inline always] get_state state = state
+let make_lazy ~f = Lazy.from_fun f
+
+let[@inline always] get_state t = Lazy.force t
 |};
   close_out oc
 ;;
