@@ -31,8 +31,9 @@
 
 open! Import
 
-type +'a t [@@deriving_inline sexp_of]
+type +'a t [@@deriving_inline globalize, sexp_of]
 
+val globalize : ('a -> 'a) -> 'a t -> 'a t
 val sexp_of_t : ('a -> Sexplib0.Sexp.t) -> 'a t -> Sexplib0.Sexp.t
 
 [@@@end]
@@ -40,7 +41,9 @@ val sexp_of_t : ('a -> Sexplib0.Sexp.t) -> 'a t -> Sexplib0.Sexp.t
 type 'a sequence := 'a t
 
 include Ppx_compare_lib.Equal.S1 with type 'a t := 'a t
+include Ppx_compare_lib.Equal.S_local1 with type 'a t := 'a t
 include Ppx_compare_lib.Comparable.S1 with type 'a t := 'a t
+include Ppx_compare_lib.Comparable.S_local1 with type 'a t := 'a t
 include Indexed_container.S1 with type 'a t := 'a t
 include Monad.S with type 'a t := 'a t
 
@@ -124,11 +127,6 @@ val filter : 'a t -> f:('a -> bool) -> 'a t
     contain duplicates. *)
 val merge_deduped_and_sorted : 'a t -> 'a t -> compare:('a -> 'a -> int) -> 'a t
 
-val merge : 'a t -> 'a t -> compare:('a -> 'a -> int) -> 'a t
-[@@deprecated
-  "[since 2021-07] For identical behavior, use [Sequence.merge_deduped_and_sorted], \
-   but consider using [Sequence.merge_sorted] instead."]
-
 (** If [t1] and [t2] are each sorted, [merge_sorted t1 t2 ~compare] merges [t1] and [t2]
     into a sorted sequence. Whenever identical elements are found in both [t1] and [t2],
     the one from [t1] is used first. The behavior is undefined if the inputs aren't
@@ -140,9 +138,12 @@ module Merge_with_duplicates_element : sig
     | Left of 'a
     | Right of 'b
     | Both of 'a * 'b
-  [@@deriving_inline compare, hash, sexp, sexp_grammar]
+  [@@deriving_inline compare ~localize, equal ~localize, hash, sexp, sexp_grammar]
 
   include Ppx_compare_lib.Comparable.S2 with type ('a, 'b) t := ('a, 'b) t
+  include Ppx_compare_lib.Comparable.S_local2 with type ('a, 'b) t := ('a, 'b) t
+  include Ppx_compare_lib.Equal.S2 with type ('a, 'b) t := ('a, 'b) t
+  include Ppx_compare_lib.Equal.S_local2 with type ('a, 'b) t := ('a, 'b) t
   include Ppx_hash_lib.Hashable.S2 with type ('a, 'b) t := ('a, 'b) t
   include Sexplib0.Sexpable.S2 with type ('a, 'b) t := ('a, 'b) t
 
@@ -174,10 +175,10 @@ val tl_eagerly_exn : 'a t -> 'a t
 
 (** [find_exn t ~f] returns the first element of [t] that satisfies [f]. It raises if
     there is no such element. *)
-val find_exn : 'a t -> f:(('a -> bool)[@local]) -> 'a
+val find_exn : 'a t -> f:('a -> bool) -> 'a
 
 (** Like [for_all], but passes the index as an argument. *)
-val for_alli : 'a t -> f:((int -> 'a -> bool)[@local]) -> bool
+val for_alli : 'a t -> f:(int -> 'a -> bool) -> bool
 
 (** [append t1 t2] first produces the elements of [t1], then produces the elements of
     [t2]. *)
@@ -220,9 +221,9 @@ val zip_full : 'a t -> 'b t -> [ `Left of 'a | `Both of 'a * 'b | `Right of 'b ]
 
 (** [reduce_exn f [a1; ...; an]] is [f (... (f (f a1 a2) a3) ...) an]. It fails on the
     empty sequence. *)
-val reduce_exn : 'a t -> f:(('a -> 'a -> 'a)[@local]) -> 'a
+val reduce_exn : 'a t -> f:('a -> 'a -> 'a) -> 'a
 
-val reduce : 'a t -> f:(('a -> 'a -> 'a)[@local]) -> 'a option
+val reduce : 'a t -> f:('a -> 'a -> 'a) -> 'a option
 
 (** [group l ~break] returns a sequence of lists (i.e., groups) whose concatenation is
     equal to the original sequence. Each group is broken where [break] returns true on a
@@ -239,10 +240,7 @@ val group : 'a t -> break:('a -> 'a -> bool) -> 'a list t
 (** [find_consecutive_duplicate t ~equal] returns the first pair of consecutive elements
     [(a1, a2)] in [t] such that [equal a1 a2].  They are returned in the same order as
     they appear in [t]. *)
-val find_consecutive_duplicate
-  :  'a t
-  -> equal:(('a -> 'a -> bool)[@local])
-  -> ('a * 'a) option
+val find_consecutive_duplicate : 'a t -> equal:('a -> 'a -> bool) -> ('a * 'a) option
 
 (** The same sequence with consecutive duplicates removed.  The relative order of the
     other elements is unaffected. *)
@@ -307,7 +305,7 @@ val drop_while : 'a t -> f:('a -> bool) -> 'a t
     alternatives would mean forcing the consumer to evaluate the first element again (if
     the previous state of the sequence is returned) or take on extra cost for each element
     (if the element is added to the final state of the sequence using [shift_right]). *)
-val drop_while_option : 'a t -> f:(('a -> bool)[@local]) -> ('a * 'a t) option
+val drop_while_option : 'a t -> f:('a -> bool) -> ('a * 'a t) option
 
 (** [split_n t n] immediately consumes the first [n] elements of [t] and returns the
     consumed prefix, as a list, along with the unevaluated tail of [t]. *)
@@ -318,7 +316,6 @@ val split_n : 'a t -> int -> 'a list * 'a t
     is not positive, it raises. *)
 val chunks_exn : 'a t -> int -> 'a list t
 
-
 (** [shift_right t a] produces [a] and then produces each element of [t]. *)
 val shift_right : 'a t -> 'a -> 'a t
 
@@ -504,4 +501,15 @@ module Expert : sig
     -> f:('s -> 'a option -> k:('s -> 'r) -> 'r) (** [k] stands for "continuation" *)
     -> finish:('s -> 'r)
     -> 'r
+
+  module View : sig
+    type +_ t = private
+      | Sequence :
+          { state : 's
+          ; next : 's -> ('a, 's) Step.t
+          }
+          -> 'a t
+  end
+
+  val view : 'a t -> 'a View.t
 end
