@@ -10,19 +10,25 @@
     replace the array starting in OCaml 4.09. *)
 val get_argv : unit -> string array
 
-(** A single result from [get_argv ()]. *)
+(** A single result from [get_argv ()]. This value is indefinitely deprecated. It is kept
+    for compatibility with {!Stdlib.Sys}. *)
 val argv : string array
-[@@deprecated
-  "[since 2019-08] Use [Sys.get_argv] instead, which has the correct behavior when \
-   [caml_sys_modify_argv] is called."]
+  [@@deprecated
+    "[since 2019-08] Use [Sys.get_argv] instead, which has the correct behavior when \
+     [caml_sys_modify_argv] is called."]
 
 (** [interactive] is set to [true] when being executed in the [ocaml] REPL, and [false]
     otherwise. *)
 val interactive : bool ref
 
-(** [os_type] describes the operating system that the OCaml program is running on.  Its
-    value is one of ["Unix"], ["Win32"], or ["Cygwin"]. When running in JavaScript, it is
-    ["Unix"]. *)
+(** [os_type] describes the operating system that the OCaml program is running on.
+
+    Its value is one of:
+    - ["Unix"] (for all Unix versions, including Linux and macOS);
+    - ["Win32"] (for MS-Windows, OCaml compiled with MSVC++ or MinGW); or
+    - ["Cygwin"] (for MS-Windows, OCaml compiled with Cygwin)
+
+    When running in JavaScript, it is ["Unix"]. *)
 val os_type : string
 
 (** [unix] is [true] if [os_type = "Unix"]. *)
@@ -112,4 +118,7 @@ val getenv_exn : string -> string
         ignore (Sys.opaque_identity (my_pure_computation ()))
       done
     ]} *)
-external opaque_identity : 'a -> 'a = "%opaque"
+external opaque_identity : ('a[@local_opt]) -> ('a[@local_opt]) = "%opaque"
+
+(** Like [opaque_identity]. Forces its argument to be globally allocated. *)
+external opaque_identity_global : 'a -> 'a = "%opaque"
