@@ -123,6 +123,35 @@ let%test_module "clamp" =
     ;;
 
     let%test "clamp bad" = Or_error.is_error (clamp 2.5 ~min:3. ~max:2.)
+    let%test "clamp also bad" = Or_error.is_error (clamp 2.5 ~min:nan ~max:3.)
+    let%test "clamp also bad 2" = Or_error.is_error (clamp 2.5 ~min:2. ~max:nan)
+    let%test "clamp also bad 3" = Or_error.is_error (clamp 2.5 ~min:nan ~max:nan)
+    let%test "clamp also bad 4" = Or_error.is_error (clamp nan ~min:nan ~max:nan)
+
+    let%test_unit "clamp_exn bad" =
+      Expect_test_helpers_base.require_does_raise [%here] (fun () ->
+        clamp_exn 2.5 ~min:3. ~max:2.)
+    ;;
+
+    let%test_unit "clamp_exn also bad" =
+      Expect_test_helpers_base.require_does_raise [%here] (fun () ->
+        clamp_exn 2.5 ~min:nan ~max:3.)
+    ;;
+
+    let%test_unit "clamp_exn also bad 2" =
+      Expect_test_helpers_base.require_does_raise [%here] (fun () ->
+        clamp_exn 2.5 ~min:2. ~max:nan)
+    ;;
+
+    let%test_unit "clamp_exn also bad 3" =
+      Expect_test_helpers_base.require_does_raise [%here] (fun () ->
+        clamp_exn 2.5 ~min:nan ~max:nan)
+    ;;
+
+    let%test_unit "clamp_exn also bad 4" =
+      Expect_test_helpers_base.require_does_raise [%here] (fun () ->
+        clamp_exn nan ~min:nan ~max:nan)
+    ;;
   end)
 ;;
 
@@ -235,7 +264,6 @@ let%test_module _ =
 
          # let x = incr 0.95 in sprintf "%.0f / %.1f / %.2f / %.3f / %.20f" x x x x x ;;
          - : string = "1 / 1.0 / 0.95 / 0.950 / 0.95000000000000006661"
-
       *)
       let f =
         if f >= 1000.
@@ -431,12 +459,12 @@ let%test_module _ =
     ;;
 
     let test_all_six
-          x
-          ~specialized_iround
-          ~specialized_iround_exn
-          ~float_rounding
-          ~dir
-          ~validate
+      x
+      ~specialized_iround
+      ~specialized_iround_exn
+      ~float_rounding
+      ~dir
+      ~validate
       =
       let result1 = iround x ~dir in
       let result2 = Option.try_with (fun () -> iround_exn x ~dir) in
@@ -446,10 +474,10 @@ let%test_module _ =
       let result6 = Option.try_with (fun () -> Int.of_float (round ~dir x)) in
       let ( = ) = Stdlib.( = ) in
       if result1 = result2
-      && result2 = result3
-      && result3 = result4
-      && result4 = result5
-      && result5 = result6
+         && result2 = result3
+         && result3 = result4
+         && result4 = result5
+         && result5 = result6
       then validate result1
       else false
     ;;
@@ -463,10 +491,10 @@ let%test_module _ =
         ~float_rounding:round_nearest
         ~dir:`Nearest
         ~validate:(function
-          | None -> true
-          | Some y ->
-            let y = of_int y in
-            -0.5 < y -. x && y -. x <= 0.5)
+        | None -> true
+        | Some y ->
+          let y = of_int y in
+          -0.5 < y -. x && y -. x <= 0.5)
     ;;
 
     (* iround_down:
@@ -483,10 +511,10 @@ let%test_module _ =
         ~float_rounding:round_down
         ~dir:`Down
         ~validate:(function
-          | None -> true
-          | Some y ->
-            let y = of_int y in
-            0. <= x -. y && x -. y < 1.)
+        | None -> true
+        | Some y ->
+          let y = of_int y in
+          0. <= x -. y && x -. y < 1.)
     ;;
 
     (* iround_up:
@@ -503,10 +531,10 @@ let%test_module _ =
         ~float_rounding:round_up
         ~dir:`Up
         ~validate:(function
-          | None -> true
-          | Some y ->
-            let y = of_int y in
-            0. <= y -. x && y -. x < 1.)
+        | None -> true
+        | Some y ->
+          let y = of_int y in
+          0. <= y -. x && y -. x < 1.)
     ;;
 
     (* iround_towards_zero:
@@ -523,11 +551,11 @@ let%test_module _ =
         ~float_rounding:round_towards_zero
         ~dir:`Zero
         ~validate:(function
-          | None -> true
-          | Some y ->
-            let x = abs x in
-            let y = abs (of_int y) in
-            0. <= x -. y && x -. y < 1. && (Sign.(sign_exn x = sign_exn y) || y = 0.0))
+        | None -> true
+        | Some y ->
+          let x = abs x in
+          let y = abs (of_int y) in
+          0. <= x -. y && x -. y < 1. && (Sign.(sign_exn x = sign_exn y) || y = 0.0))
     ;;
 
     (* Easy cases that used to live inline with the code above. *)
@@ -801,14 +829,14 @@ let%test_module _ =
 ;;
 
 module Test_bounds (I : sig
-    type t
+  type t
 
-    val num_bits : int
-    val of_float : float -> t
-    val to_int64 : t -> Int64.t
-    val max_value : t
-    val min_value : t
-  end) =
+  val num_bits : int
+  val of_float : float -> t
+  val to_int64 : t -> Int64.t
+  val max_value : t
+  val min_value : t
+end) =
 struct
   open I
 
@@ -836,7 +864,7 @@ struct
     in
     let min_value = to_int64 min_value in
     if Int.( = ) num_bits 64
-    (* We cannot detect overflow because on Intel overflow results in min_value. *)
+       (* We cannot detect overflow because on Intel overflow results in min_value. *)
     then true
     else (
       assert (Int64.( <= ) lower_bound_minus_epsilon lower_bound);
@@ -851,7 +879,7 @@ struct
     in
     let max_value = to_int64 max_value in
     if Int.( = ) num_bits 64
-    (* upper_bound_plus_epsilon is not representable as a Int64.t, it has overflowed *)
+       (* upper_bound_plus_epsilon is not representable as a Int64.t, it has overflowed *)
     then Int64.( < ) upper_bound_plus_epsilon upper_bound
     else (
       assert (Int64.( >= ) upper_bound_plus_epsilon upper_bound);
@@ -962,7 +990,6 @@ let%test_module _ =
 
 let%test _ = round_nearest 3.6 = 4. && round_nearest (-3.6) = -4.
 
-
 (* The redefinition of [sexp_of_t] in float.ml assumes sexp conversion uses E rather than
    e. *)
 let%test_unit "e vs E" =
@@ -1054,7 +1081,8 @@ let%expect_test "square" =
   printf "%f\n" (square (-2.5));
   [%expect {|
     2.250000
-    6.250000 |}]
+    6.250000
+    |}]
 ;;
 
 let%expect_test "mathematical constants" =
@@ -1070,7 +1098,8 @@ let%expect_test "mathematical constants" =
   [%expect
     {|
     sqrt pi diff  : 0.00000000000000022204
-    sqrt 2pi diff : 0.00000000000000044409 |}]
+    sqrt 2pi diff : 0.00000000000000044409
+    |}]
 ;;
 
 let%test _ = not (is_negative Float.nan)
@@ -1162,7 +1191,8 @@ let%expect_test "is_nan, is_inf, and is_finite" =
        4.94065645841247e-324 false false  true
                           1. false false  true
      1.7976931348623157e+308 false false  true
-                         inf false  true false |}]
+                         inf false  true false
+    |}]
 ;;
 
 let%expect_test "nan" =
@@ -1186,3 +1216,92 @@ let%expect_test "iround_exn" =
   require_equal [%here] (module Int) 3 (Float.iround_exn ~dir:`Nearest 3.4);
   require_equal [%here] (module Int) (-3) (Float.iround_exn ~dir:`Nearest (-3.4))
 ;;
+
+let%expect_test "log" =
+  let test float =
+    let log2 = Float.log2 float in
+    let log10 = Float.log10 float in
+    let ratio = log2 /. log10 in
+    let ratio =
+      (* NAN behavior differs in js_of_ocaml *)
+      if Float.is_nan ratio
+      then Float.nan
+      else (
+        assert (Float.(abs ratio - (1. / log10 2.) < 1e-15));
+        ratio)
+    in
+    print_s [%sexp { log2 : float; log10 : float; ratio : float }]
+  in
+  test (-1.);
+  [%expect {|
+    ((log2  NAN)
+     (log10 NAN)
+     (ratio NAN))
+    |}];
+  test 0.;
+  [%expect {|
+    ((log2  -INF)
+     (log10 -INF)
+     (ratio NAN))
+    |}];
+  test 1.;
+  [%expect {|
+    ((log2  0)
+     (log10 0)
+     (ratio NAN))
+    |}];
+  test 2.;
+  [%expect
+    {|
+    ((log2  1)
+     (log10 0.3010299956639812)
+     (ratio 3.3219280948873622))
+    |}];
+  test 10.;
+  [%expect
+    {|
+    ((log2  3.3219280948873622)
+     (log10 1)
+     (ratio 3.3219280948873622))
+    |}];
+  test Float.min_positive_subnormal_value;
+  [%expect
+    {|
+    ((log2  -1074)
+     (log10 -323.30621534311581)
+     (ratio 3.3219280948873622))
+    |}];
+  test Float.epsilon_float;
+  [%expect
+    {|
+    ((log2  -52)
+     (log10 -15.653559774527022)
+     (ratio 3.3219280948873626))
+    |}];
+  test Float.pi;
+  [%expect
+    {|
+    ((log2  1.6514961294723187)
+     (log10 0.4971498726941338)
+     (ratio 3.3219280948873626))
+    |}];
+  test Float.max_finite_value;
+  [%expect
+    {|
+    ((log2  1024)
+     (log10 308.25471555991675)
+     (ratio 3.3219280948873622))
+    |}];
+  test Float.infinity;
+  [%expect {|
+    ((log2  INF)
+     (log10 INF)
+     (ratio NAN))
+    |}]
+;;
+
+let%expect_test "float comparisons permit both local and global arguments" =
+  let (_ : float -> float -> bool) = Float.( < ) in
+  let (_ : float -> float -> bool) = Float.( < ) in
+  [%expect {| |}]
+;;
