@@ -18,7 +18,8 @@ let%expect_test "Finished_or_unfinished <-> Continue_or_stop" =
       (Finished_or_unfinished.of_continue_or_stop c_or_s));
   [%expect {|
     (Continue Finished)
-    (Stop Unfinished) |}]
+    (Stop Unfinished)
+    |}]
 ;;
 
 let%test _ =
@@ -55,10 +56,11 @@ let%expect_test "[Map.of_alist_multi] preserves value ordering" =
   print_s
     [%sexp
       (Map.of_alist_multi (module String) [ "a", 1; "a", 2; "b", 1; "b", 3 ]
-       : int list Map.M(String).t)];
+        : int list Map.M(String).t)];
   [%expect {|
     ((a (1 2))
-     (b (1 3))) |}]
+     (b (1 3)))
+    |}]
 ;;
 
 let%expect_test "find_exn" =
@@ -108,21 +110,24 @@ let%expect_test "combine_errors" =
   test [ Error "one" ];
   [%expect {|
     (Ok ((1 one)))
-    (Error ((1 one))) |}];
+    (Error ((1 one)))
+    |}];
   (* multiple ok *)
   test [ Ok "one"; Ok "two"; Ok "three" ];
   [%expect {|
     (Ok (
       (1 one)
       (2 two)
-      (3 three))) |}];
+      (3 three)))
+    |}];
   (* multiple errors *)
   test [ Error "one"; Error "two"; Error "three" ];
   [%expect {|
     (Error (
       (1 one)
       (2 two)
-      (3 three))) |}];
+      (3 three)))
+    |}];
   (* one error among oks *)
   test [ Error "one"; Ok "two"; Ok "three" ];
   test [ Ok "one"; Error "two"; Ok "three" ];
@@ -130,7 +135,8 @@ let%expect_test "combine_errors" =
   [%expect {|
     (Error ((1 one)))
     (Error ((2 two)))
-    (Error ((3 three))) |}];
+    (Error ((3 three)))
+    |}];
   (* one ok among errors *)
   test [ Ok "one"; Error "two"; Error "three" ];
   test [ Error "one"; Ok "two"; Error "three" ];
@@ -145,7 +151,8 @@ let%expect_test "combine_errors" =
       (3 three)))
     (Error (
       (1 one)
-      (2 two))) |}]
+      (2 two)))
+    |}]
 ;;
 
 let%test_module "Poly" =
@@ -253,13 +260,13 @@ let%test_module "[symmetric_diff]" =
         List.map map_pairs ~f:(fun (m, m') ->
           measure_comparisons (fun () ->
             diffs
-            := !diffs
-               + Map.fold_symmetric_diff
-                   ~init:0
-                   ~f:(fun n _ -> n + 1)
-                   ~data_equal:(fun () () -> true)
-                   (m : unit Map.M(Key).t)
-                   m'))
+              := !diffs
+                 + Map.fold_symmetric_diff
+                     ~init:0
+                     ~f:(fun n _ -> n + 1)
+                     ~data_equal:(fun () () -> true)
+                     (m : unit Map.M(Key).t)
+                     m'))
       in
       let worst_counts =
         List.sort counts ~compare:[%compare: int] |> List.rev |> fun l -> List.take l 20
@@ -359,7 +366,7 @@ let%expect_test "[map_keys]" =
     print_s
       [%sexp
         (Map.map_keys c ~f m
-         : [ `Duplicate_key of string | `Ok of string Map.M(String).t ])]
+          : [ `Duplicate_key of string | `Ok of string Map.M(String).t ])]
   in
   let map = Map.of_alist_exn (module Int) [ 1, "one"; 2, "two"; 3, "three" ] in
   test map (module String) ~f:Int.to_string;
@@ -367,7 +374,8 @@ let%expect_test "[map_keys]" =
     (Ok (
       (1 one)
       (2 two)
-      (3 three))) |}];
+      (3 three)))
+    |}];
   test map (module String) ~f:(fun x -> Int.to_string (x / 2));
   [%expect {| (Duplicate_key 1) |}]
 ;;
@@ -381,7 +389,7 @@ let%expect_test "[fold_until]" =
            ~init:0
            ~f:(fun ~key ~data acc -> if key > 2 then Stop data else Continue (acc + key))
            ~finish:Int.to_string
-         : string)]
+          : string)]
   in
   let map = Map.of_alist_exn (module Int) [ 1, "one"; 2, "two"; 3, "three" ] in
   test map;
@@ -390,3 +398,37 @@ let%expect_test "[fold_until]" =
   test map;
   [%expect {| 2 |}]
 ;;
+
+let%expect_test "[sum]" =
+  let test t = print_s [%sexp (Map.sum (module Int) t ~f:(( * ) 2) : int)] in
+  let map = Map.of_alist_exn (module String) [ "A", 1; "B", 2; "C", 3 ] in
+  test map;
+  [%expect {| 12 |}]
+;;
+
+let%expect_test "[sumi]" =
+  let test t =
+    print_s [%sexp (Map.sumi (module Int) t ~f:(fun ~key ~data -> key * data) : int)]
+  in
+  let map = Map.of_alist_exn (module Int) [ 1, 1; 2, 2; 3, 3 ] in
+  test map;
+  [%expect {| 14 |}]
+;;
+
+let%expect_test "[merge_disjoint_exn] success" =
+  let map1 = Map.of_alist_exn (module Int) [ 1, "one"; 2, "two" ] in
+  let map2 = Map.of_alist_exn (module Int) [ 3, "three" ] in
+  print_s [%sexp (Map.merge_disjoint_exn map1 map2 : string Map.M(Int).t)];
+  [%expect {|
+    ((1 one)
+     (2 two)
+     (3 three))
+    |}]
+;;
+
+let%expect_test "[merge_disjoint_exn] failure" =
+  let map1 = Map.of_alist_exn (module Int) [ 1, "one"; 2, "two" ] in
+  let map2 = Map.of_alist_exn (module Int) [ 2, "two"; 3, "three" ] in
+  show_raise (fun () -> Map.merge_disjoint_exn map1 map2);
+  [%expect {| (raised ("Map.merge_disjoint_exn: duplicate key" 2)) |}]
+;;
